@@ -1,5 +1,5 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
 
 #include <cstdio>
@@ -24,16 +24,48 @@
 #include "blockmap.h"
 
 
-// Return the area of this block only (no recursive)
+// Return the total filled area of this block (no recursive)
 int Block::area() const throw()
   {
-  int area = 0;
+  int a = 0;
 
   for( int row = top(); row <= bottom(); ++row )
     for( int col = left(); col <= right(); ++col )
-      if( _blockmap->id( row, col ) == _id ) ++area;
+      if( _blockmap->id( row, col ) == _id ) ++a;
 
-  return area;
+  return a;
+  }
+
+
+// Return the central octagon filled area of this block (no recursive)
+int Block::area_octagon() const throw()
+  {
+  int a = 0;
+  int bevel = ( 29 * std::min( height(), width() ) ) / 100;
+  int l =  left() + bevel;
+  int r = right() - bevel;
+
+  for( int i = 0; i < bevel; ++i )
+    for( int row = top() + i, col = l - i; col <= r + i; ++col )
+      if( _blockmap->id( row, col ) == _id ) ++a;
+
+  for( int row = top() + bevel; row <= bottom() - bevel; ++row )
+    for( int col = left(); col <= right(); ++col )
+      if( _blockmap->id( row, col ) == _id ) ++a;
+
+  for( int i = bevel - 1; i >= 0; --i )
+    for( int row = bottom() - i, col = l - i; col <= r + i; ++col )
+      if( _blockmap->id( row, col ) == _id ) ++a;
+
+  return a;
+  }
+
+
+const Block & Block::block( int i ) const throw()
+  {
+  if( i < 0 || i >= blocks() )
+    Ocrad::internal_error( "block, index out of bounds" );
+  return _block_vector[i];
   }
 
 
@@ -230,31 +262,6 @@ int Block::follow_bottom( int row, int col ) const throw()
   return row - 1;
   }
 
-/*
-bool Block::seek_around( int *row, int *col, bool black ) const throw()
-  {
-  const Blockmap & bm = *_blockmap;
-  if( ( bm.id( *row, *col ) == _id ) == black ) return true;
-  bool d[3][3] = { { true,  true, true },
-                   { true, false, true },
-                   { true,  true, true } };
-  for( int i = 0 ; i < 3; ++i )
-    for( int j = 0 ; j < 3; ++j )
-      if( d[i][j] )
-        if( ( bm.id( *row + i - 1, *col + i - 1 ) == _id ) != black )
-          d[i][j] = false;
-
-  if( d[0][1] ) { --(*row); return true; }
-  if( d[2][1] ) { ++(*row); return true; }
-  if( d[1][0] ) { --(*col); return true; }
-  if( d[1][2] ) { ++(*row); return true; }
-  if( d[0][0] ) { --(*row); --(*col); return true; }
-  if( d[0][2] ) { --(*row); ++(*col); return true; }
-  if( d[2][0] ) { ++(*row); --(*col); return true; }
-  if( d[2][2] ) { ++(*row); ++(*col); return true; }
-  return false;
-  }
-*/
 
 int Block::seek_left( int row, int col, bool black ) const throw()
   {
@@ -301,19 +308,27 @@ int Block::seek_bottom( int row, int col, bool black ) const throw()
 bool Block::top_hook( int *hdiff ) const throw()
   {
   const Blockmap & bm = *_blockmap;
-  int row, lcol = 0, rcol = 0, black_section = 0;
+  int row, lcol = 0, rcol = 0, black_section = 0, wmax = 0;
 
   for( row = top() + 1; row < vcenter(); ++row )
     {
+    int l = -1, r = -2;
     bool prev_black = false;
     black_section = 0;
     for( int col = left(); col <= right(); ++col )
       {
       bool black = ( bm.id( row, col ) == _id );
-      if( black ) { if( !prev_black && ++black_section == 2 ) rcol = col; }
+      if( black )
+        {
+        if( !prev_black && ++black_section == 2 ) rcol = col;
+        r = col; if( l < 0 ) l = col;
+        }
       else if( prev_black && black_section == 1 ) lcol = col - 1;
       prev_black = black;
       }
+    r = r - l + 1;
+    if( 10 * r <= 9 * wmax ) return false;
+    if( r > wmax ) wmax = r ;
     if( black_section >= 2 ) break;
     }
 
@@ -331,19 +346,27 @@ bool Block::top_hook( int *hdiff ) const throw()
 bool Block::bottom_hook( int *hdiff ) const throw()
   {
   const Blockmap & bm = *_blockmap;
-  int row, lcol = 0, rcol = 0, black_section = 0;
+  int row, lcol = 0, rcol = 0, black_section = 0, wmax = 0;
 
   for( row = bottom() - 1; row > vcenter(); --row )
     {
+    int l = -1, r = -2;
     bool prev_black = false;
     black_section = 0;
     for( int col = left(); col <= right(); ++col )
       {
       bool black = ( bm.id( row, col ) == _id );
-      if( black ) { if( !prev_black && ++black_section == 2 ) rcol = col; }
+      if( black )
+        {
+        if( !prev_black && ++black_section == 2 ) rcol = col;
+        r = col; if( l < 0 ) l = col;
+        }
       else if( prev_black && black_section == 1 ) lcol = col - 1;
       prev_black = black;
       }
+    r = r - l + 1;
+    if( 10 * r <= 8 * wmax ) return false;
+    if( r > wmax ) wmax = r ;
     if( black_section >= 2 ) break;
     }
 
@@ -379,6 +402,25 @@ void Block::print( FILE * outfile, int sp ) const throw()
   }
 
 
+// Returns true if height is or can be adjusted
+bool Block::adjust_height() throw()
+  {
+  int row1, row2;
+
+  for( row1 = top(); row1 <= bottom(); ++row1 )
+    for( int col = left(); col <= right(); ++col )
+      if( _blockmap->id( row1, col ) == _id ) goto L1;
+  L1:
+  for( row2 = bottom(); row2 >= row1; --row2 )
+    for( int col = left(); col <= right(); ++col )
+      if( _blockmap->id( row2, col ) == _id ) goto L2;
+  L2:
+  if( row1 > top() || row2 < bottom() )
+    { if( row1 <= row2 ) { top( row1 ); bottom( row2 ); } else return false; }
+  return true;
+  }
+
+
 void Block::shift_block( const Block & b ) throw()
   {
   int i = blocks() - 1;
