@@ -1,5 +1,5 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
 
 #include <cstdio>
@@ -88,7 +88,7 @@ void remove_top_bottom_noise( std::vector< Block > & block_vector ) throw()
   for( int i = 0; i < blocks; ++i )
     {
     Block & b = block_vector[i];
-    if( b.height() <= 10 ) continue;
+    if( b.height() < 11 ) continue;
     const Blockmap & bm = *(b.blockmap());
 
     int c = 0;
@@ -103,6 +103,28 @@ void remove_top_bottom_noise( std::vector< Block > & block_vector ) throw()
     }
   }
 
+
+void remove_left_right_noise( std::vector< Block > & block_vector ) throw()
+  {
+  int blocks = block_vector.size();
+  for( int i = 0; i < blocks; ++i )
+    {
+    Block & b = block_vector[i];
+    if( b.width() < 6 ) continue;
+    const Blockmap & bm = *(b.blockmap());
+
+    int c = 0;
+    for( int row = b.top(); row <= b.bottom(); ++row )
+      if( bm.id( row, b.left() ) == b.id() && ++c > 1 ) break;
+    if( c <= 1 ) b.left( b.left() + 1 );
+
+    c = 0;
+    for( int row = b.top(); row <= b.bottom(); ++row )
+      if( bm.id( row, b.right() ) == b.id() && ++c > 1 ) break;
+    if( c <= 1 ) b.right( b.right() - 1 );
+    }
+  }
+
 } // end namespace
 
 
@@ -184,7 +206,7 @@ void Blockmap::ignore_wide_blocks() throw()
     {
     const Block & b = _block_vector[i];
     if( 2 * b.width() < _width ) { ++i; continue; }
-    if( b.id() > 0 && 4 * b.area() > b.width() * b.height() )
+    if( b.id() > 0 && 4 * b.area() > b.size() )
       {
       // image, not frame
       if( 10 * b.width() > 8 * _width && 10 * b.height() > 8 * _height )
@@ -278,6 +300,7 @@ Blockmap::Blockmap( const Bitmap & page_image, int rindex, int debug_level )
     Block::hierarchize_blocks( _block_vector );
     ignore_white_blocks( _block_vector );
     remove_top_bottom_noise( _block_vector );
+    remove_left_right_noise( _block_vector );
     if( debug_level <= 97 ) order_blocks( _block_vector );
     }
   }
