@@ -1,5 +1,5 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -13,25 +13,33 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
 
 #include <cstdio>
 #include <vector>
 #include "common.h"
 #include "rectangle.h"
+#include "ucs.h"
 #include "block.h"
 #include "blockmap.h"
 #include "character.h"
-#include "iso_8859_1.h"
 
 
-// Return the area of the main blocks only (no recursive)
+// Return the filled area of the main blocks only (no recursive)
 int Character::area() const throw()
   {
-  int area = 0;
-  for( int i = 0; i < blocks(); ++i ) area += _block_vector[i].area();
-  return area;
+  int a = 0;
+  for( int i = 0; i < blocks(); ++i ) a += _block_vector[i].area();
+  return a;
+  }
+
+
+const Block & Character::block( int i ) const throw()
+  {
+  if( i < 0 || i >= blocks() )
+    Ocrad::internal_error( "block, index out of bounds" );
+  return _block_vector[i];
   }
 
 
@@ -51,17 +59,17 @@ void Character::shift_block( const Block & b ) throw()
   }
 
 
-void Character::add_guess( unsigned char ch, int value ) throw()
+void Character::add_guess( int code, int value ) throw()
   {
-  _guess.push_back( Guess( ch, value ) );
+  _guess.push_back( Guess( code, value ) );
   }
 
 
-void Character::insert_guess( int i, unsigned char ch, int value ) throw()
+void Character::insert_guess( int i, int code, int value ) throw()
   {
   if( i < 0 || i > guesses() )
     Ocrad::internal_error( "insert_guess, index out of bounds" );
-  _guess.insert( _guess.begin() + i, Guess( ch, value ) );
+  _guess.insert( _guess.begin() + i, Guess( code, value ) );
   }
 
 
@@ -73,16 +81,16 @@ void Character::delete_guess( int i ) throw()
   }
 
 
-void Character::only_guess( unsigned char ch, int value ) throw()
-  { _guess.clear(); add_guess( ch, value ); }
+void Character::only_guess( int code, int value ) throw()
+  { _guess.clear(); add_guess( code, value ); }
 
 
 void Character::swap_guesses( int i, int j ) throw()
   {
   if( i < 0 || i >= guesses() || j < 0 || j >= guesses() )
     Ocrad::internal_error( "swap_guesses, index out of bounds" );
-  unsigned char ch = _guess[i].ch;
-  _guess[i].ch = _guess[j].ch; _guess[j].ch = ch;
+  int code = _guess[i].code;
+  _guess[i].code = _guess[j].code; _guess[j].code = code;
   }
 
 
@@ -94,6 +102,14 @@ const Character::Guess & Character::guess( int i ) const throw()
   }
 
 
+bool Character::maybe( int code ) const throw()
+  {
+  for( int i = 0; i < guesses(); ++i )
+    if( code == _guess[i].code ) return true;
+  return false;
+  }
+
+
 void Character::join( Character & c ) throw()
   {
   if( _blockmap != c._blockmap )
@@ -102,22 +118,49 @@ void Character::join( Character & c ) throw()
   }
 
 
-void Character::print( FILE * outfile, bool graph, bool recursive )
-							const throw()
+void Character::print( const Control & control ) const throw()
+  {
+  if( guesses() )
+    switch( control.format )
+      {
+      case Control::byte:
+        { char ch = UCS::map_to_byte( _guess[0].code );
+        if( ch ) std::putc( ch, control.outfile ); }
+        break;
+      case Control::utf8:
+        std::fputs( UCS::ucs_to_utf8( _guess[0].code ), control.outfile );
+      }
+  else std::putc( '_', control.outfile );
+  }
+
+
+void Character::dprint( const Control & control, bool graph, bool recursive )
+								const throw()
   {
   if( graph || recursive )
+    std::fprintf( control.outfile, "%d guesses    ", guesses() );
+  for( int i = 0; i < guesses(); ++i )
     {
-    std::fprintf( outfile, "%d guesses    ", guesses() );
-    for( int i = 0; i < guesses(); ++i )
-      std::fprintf( outfile, "guess '%c', confidence %d    ",
-                    _guess[i].ch, _guess[i].value );
-    std::fputs( "\n", outfile );
+    switch( control.format )
+      {
+      case Control::byte:
+        { char ch = UCS::map_to_byte( _guess[i].code );
+        if( ch ) std::fprintf( control.outfile, "guess '%c', confidence %d    ",
+                               ch, _guess[i].value ); }
+        break;
+      case Control::utf8:
+        std::fprintf( control.outfile, "guess '%s', confidence %d    ",
+                      UCS::ucs_to_utf8( _guess[i].code ), _guess[i].value );
+      }
+    if( !graph && !recursive ) break;
     }
+  std::fputs( "\n", control.outfile );
   if( graph )
     {
-    std::fprintf( outfile, "width = %d, height = %d, hcenter = %d, vcenter = %d, black area = %d%%\n",
+    std::fprintf( control.outfile,
+                  "width = %d, height = %d, hcenter = %d, vcenter = %d, black area = %d%%\n",
                   width(), height(), hcenter(), vcenter(),
-                  ( area() * 100 ) / ( height() * width() ) );
+                  ( area() * 100 ) / size() );
     for( int row = top(); row <= bottom(); ++row)
       {
       for( int col = left(); col <= right(); ++col)
@@ -135,33 +178,31 @@ void Character::print( FILE * outfile, bool graph, bool recursive )
               break;
               }
           }
-        std::fprintf( outfile, " %c", ch );
+        std::fprintf( control.outfile, " %c", ch );
         }
-      std::fputs( "\n", outfile );
+      std::fputs( "\n", control.outfile );
       }
-    std::fputs( "\n", outfile );
-    }
-  else
-    {
-    if( recursive )
-      {
-      for( int i = 0; i < guesses(); ++i )
-        std::fprintf( outfile, "%c %4d,    ", _guess[i].ch, _guess[i].value );
-      std::fputs( "\n", outfile );
-      }
-    else if( guesses() ) std::putc( _guess[0].ch, outfile );
-    else std::putc( '_', outfile );
+    std::fputs( "\n", control.outfile );
     }
   }
 
 
-void Character::xprint( FILE * exportfile ) const throw()
+void Character::xprint( const Control & control ) const throw()
   {
-  std::fprintf( exportfile, "%3d %3d %2d %2d; %d",
+  std::fprintf( control.exportfile, "%3d %3d %2d %2d; %d",
                 left(), top(), width(), height(), guesses() );
 
   for( int i = 0; i < guesses(); ++i )
-    std::fprintf( exportfile, ", '%c'%d", _guess[i].ch, _guess[i].value );
-
-  std::fputs( "\n", exportfile );
+    switch( control.format )
+      {
+      case Control::byte:
+        { char ch = UCS::map_to_byte( _guess[i].code );
+        if( !ch ) ch = '_';
+        std::fprintf( control.exportfile, ", '%c'%d", ch, _guess[i].value ); }
+        break;
+      case Control::utf8:
+        std::fprintf( control.exportfile, ", '%s'%d",
+                      UCS::ucs_to_utf8( _guess[i].code ), _guess[i].value );
+      }
+  std::fputs( "\n", control.exportfile );
   }
