@@ -1,5 +1,5 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -13,29 +13,29 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
 
 class Character : public Rectangle
   {
   struct Guess
     {
-    unsigned char ch;
+    int code;
     int value;
-    Guess( unsigned char c, int v ) throw() : ch( c ), value( v ) {}
+    Guess( int c, int v ) throw() : code( c ), value( v ) {}
     };
 
+  const Blockmap * _blockmap;	// Blockmap to which this character belongs
   std::vector< Block > _block_vector; // the blocks forming this Character
   std::vector< Guess > _guess;	// vector of possible char codes
 				// and their associated values
-  const Blockmap * _blockmap;	// Blockmap to which this character belongs
 
-  void recognize11( int charbox_vcenter ) throw();
-    void recognize110( int charbox_vcenter ) throw();
-    void recognize111() throw();
+  void recognize11( const Charset & charset, int charbox_vcenter ) throw();
+    void recognize110( const Charset & charset, int charbox_vcenter ) throw();
+    void recognize111( const Charset & charset, int charbox_vcenter ) throw();
     void recognize112() throw();
-  void recognize12( int charbox_vcenter ) throw();
-  void recognize13() throw();
+  void recognize12( const Charset & charset, int charbox_vcenter ) throw();
+  void recognize13( const Charset & charset ) throw();
 
 public:
   Character( const Block & b ) throw()
@@ -45,27 +45,26 @@ public:
   Character( const Rectangle & r, const Blockmap & b ) throw()
     : Rectangle( r ), _blockmap( &b ) {}
 
-  int area() const throw();
-  const std::vector< Block > & block_vector() const throw()
-    { return _block_vector; }
+  int   area() const throw();
+  const Block & block( int i ) const throw();
+  int   blocks() const throw() { return _block_vector.size(); }
   const Blockmap * blockmap() const throw() { return _blockmap; }
-  int blocks() const throw() { return _block_vector.size(); }
 
-  void only_block( const Block & b ) throw()
-    { _block_vector.clear(); _block_vector.push_back( b ); }
   void shift_block( const Block & b ) throw();
 
-  void add_guess( unsigned char ch, int value ) throw();
-  void insert_guess( int i, unsigned char ch, int value ) throw();
+  void add_guess( int code, int value ) throw();
+  void insert_guess( int i, int code, int value ) throw();
   void delete_guess( int i ) throw();
-  void only_guess( unsigned char ch, int value ) throw();
+  void only_guess( int code, int value ) throw();
   void swap_guesses( int i, int j ) throw();
   const Guess & guess( int i ) const throw();
   int guesses() const throw() { return _guess.size(); }
+  bool maybe( int code ) const throw();
 
   void join( Character & c ) throw();
-  void print( FILE * outfile, bool graph, bool recursive ) const throw();
-  void xprint( FILE * exportfile ) const throw();
+  void print( const Control & control ) const throw();
+  void dprint( const Control & control, bool graph, bool recursive ) const throw();
+  void xprint( const Control & control ) const throw();
 
-  void recognize1( int charbox_vcenter ) throw();
+  void recognize1( const Charset & charset, int charbox_vcenter ) throw();
   };
