@@ -1,5 +1,5 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
 
 #include <algorithm>
@@ -21,11 +21,11 @@
 #include <vector>
 #include "common.h"
 #include "rectangle.h"
+#include "ucs.h"
 #include "block.h"
 #include "blockmap.h"
 #include "profile.h"
 #include "feats.h"
-#include "iso_8859_1.h"
 
 
 Features::Features( const Block & b ) throw()
@@ -126,7 +126,7 @@ int Features::vbars() const throw()	// FIXME small gaps not detected
                 break;
         case 3: if( count * 3 < limit * 2 || col == b.right() )
                   {
-                  int end = ( col == b.right() ) ? col : col - 1;
+                  int end = ( count * 3 < limit * 2 ) ? col - 1 : col;
                   _vbar.push_back( Rectangle( begin, b.top(), end, b.bottom() ) );
                   ++_vbars; state = 0;
                   }
@@ -197,7 +197,7 @@ const std::vector< int > & Features::vscan() const throw()
 
 // Looks for three black sections in column hcenter()  1, then tests if
 // upper and lower gaps are open to the right or to the left
-unsigned char Features::test_235Esz() const throw()
+int Features::test_235Esz( const Charset & charset ) const throw()
   {
   static const int csize = 5;
   static const int ucoff[csize] = { 0, +1, -1, -1, +1 };
@@ -246,12 +246,20 @@ unsigned char Features::test_235Esz() const throw()
       if( b.height() > 3 * b.width() ) return 0;
       return '3';
       }
-    if( b.escape_right( urow2, ucol2 ) )
+    else if( b.escape_right( urow2, ucol2 ) )
       {
-      if( b.seek_right( lrow1 + 1, lcol1 ) >= b.right() ) return '5';
-      if( urow2 > b.vcenter() + 1 && lrow2 > b.bottom() - (b.height() / 5) &&
-          b.seek_right( urow2 - 1, ucol2 ) < b.right() )
-        return ISO_8859_1::SCCEDI;
+      if( b.seek_right( lrow1 + 1, lcol1 ) >= b.right() &&
+          b.seek_left( urow3 - 1, ucol3 ) > b.left() )
+        {
+        int hdiff;
+        if( !b.top_hook( &hdiff ) || 5 * hdiff >= 4 * b.height() )
+          return '5';
+        }
+      if( charset.enabled( Charset::iso_8859_15 ) ||
+          charset.enabled( Charset::iso_8859_9 ) )
+        if( urow2 > b.vcenter() + 1 &&
+            b.seek_right( urow2 - 1, ucol2 ) < b.right() )
+          return UCS::SCCEDI;
       return 's';
       }
     }
@@ -264,16 +272,21 @@ unsigned char Features::test_235Esz() const throw()
       if( bp.minima( b.height() / 5 ) == 1 )
         {
         if( b.escape_bottom( urow3, ucol3 ) )
-          { if( 2 * lp[lp.samples()-2] > rp[rp.samples()-2] ) return '';
-          return 'F'; }
-        if( b.escape_top( urow2, b.left() ) &&
+          {
+          if( charset.enabled( Charset::iso_8859_15 ) ||
+              charset.enabled( Charset::iso_8859_9 ) )
+            if( 2 * lp[lp.samples()-2] > rp[rp.samples()-2] )
+              return UCS::CCCEDI;
+          return 'F';
+          }
+        if( 8 * lp[((lrow2+urow3)/2)-b.top()] >= b.width() &&
+            b.escape_top( urow2, b.left() ) &&
             !b.escape_top( lrow2, b.left() ) ) return 'f';
-        else return 'E';
+        else if( lrow1 < urow2 && lrow2 < urow3 ) return 'E';
         }
       }
-    if( b.escape_left( urow2, ucol2 ) )
+    else if( b.escape_left( urow2, ucol2 ) )
       {
-      if( tp.isflat() ) return 'z';
       if( ( tp.isconvex() || ( tp.ispit() && !bp.ispit() ) ) &&
           lp[lp.pos(50)] >= b.width() / 2 ) return '2';
       if( b.height() > 2 * wp.max() && bp.minima( b.height() / 5 ) == 1 )
@@ -285,16 +298,50 @@ unsigned char Features::test_235Esz() const throw()
   }
 
 
+// Looks for four black sections in column hcenter()  1, then tests if
+// upper gap is open to the right and lower gaps are open to the left
+int Features::test_s_cedilla() const throw()
+  {
+  const Block & b = *_block;
+  const Blockmap & bm = *_blockmap;
+  int urow2 = 0, urow3 = 0, urow4 = 0, col, black_section = 0;
+
+  for( col = b.hcenter() - 1; col <= b.hcenter() + 1; ++col )
+    {
+    bool prev_black = false;
+    for( int row = b.top(); row <= b.bottom(); ++row )
+      {
+      bool black = ( bm.id( row, col ) == b.id() );
+      if( black && !prev_black )
+        {
+        if( ++black_section == 2 ) urow2 = row - 1;
+        else if( black_section == 3 ) urow3 = row - 1;
+        else if( black_section == 4 ) urow4 = row - 1;
+        }
+      prev_black = black;
+      }
+    if( black_section == 4 && urow2 < b.vpos(50) && urow4 >= b.vpos(70) )
+      break;
+    black_section = 0;
+    }
+
+  if( black_section == 4 && b.escape_right( urow2, col ) &&
+      b.escape_left( urow3, col ) && b.escape_left( urow4, col ) )
+    return UCS::SSCEDI;
+  return 0;
+  }
+
+
 // Tests if the lower half of character is open to the left, to the right,
 // and/or to the bottom
-unsigned char Features::test_49ARegpq() const throw()
+int Features::test_49ARegpq() const throw()
   {
   const Block & b = *_block;
 
-  if( bp.minima( b.height() / 10 ) == 2 && bp.isctip() )
+  if( bp.minima( b.height() / 10 ) == 2 && bp.isctip() && tp.minima() == 1 )
     { if( tp.isvpit() ) return 'A'; else return 'R'; }
 
-  const Block & h = b.block_vector().front();		// hole
+  const Block & h = b.block( 0 );			// hole
   int col = h.hcenter();
   int row = b.seek_bottom( h.bottom(), col, false ) + 1;
   if( row >= b.vpos(90) )
@@ -312,7 +359,7 @@ unsigned char Features::test_49ARegpq() const throw()
       if( row2 < b.vpos(75) ) return 'g';
       if( row2 < b.bottom() ) return 'e';
       }
-    return 'p';
+    if( bp.minima() == 1 ) return 'p';
     }
 
   if( b.escape_left( row, col ) )
@@ -326,7 +373,7 @@ unsigned char Features::test_49ARegpq() const throw()
       return '9';
     int hdiff;
     if( b.bottom_hook( &hdiff ) && hdiff > 0 )
-      { if( rp.isctip() ) return 's'; return 'g'; }
+      { if( rp.isctip() ) return 's'; else return 'g'; }
     if( row > b.vpos(80) ) return 'Q';
     if( b.seek_bottom( row, col ) < b.bottom() &&
         rp.increasing( row - b.top() ) ) return 'g';
@@ -336,21 +383,25 @@ unsigned char Features::test_49ARegpq() const throw()
   }
 
 
-unsigned char Features::test_4ADQao() const throw()
+int Features::test_4ADQao( const Charset & charset, int charbox_vcenter ) const throw()
   {
   const Block & b = *_block;
-  const Block & h = b.block_vector().front();
+  const Block & h = b.block( 0 );			// hole
+  int left_delta = h.left() - b.left(), right_delta = b.right() - h.right();
 
-  if( lp.isflats() && !rp.isflats() ) return 'D';
+  if( lp.isflats() && rp.ispit() ) return 'D';
   if( !rp.isconvex() )
     {
-    if( tp.minima() == 2 ) return '#';
+    if( Ocrad::similar( left_delta, right_delta, 40 ) &&
+        tp.minima() == 2 && bp.minima() == 2 ) return '#';
     if( tp.minima() == 1 && bp.minima() == 1 )
       {
       int row = b.seek_bottom( h.bottom(), h.hcenter(), false );
-      if( !lp.isconvex() && bp.isconvex() &&
-          b.seek_bottom( row, h.hcenter() ) < b.bottom() )
-        return ISO_8859_1::SEACUTE;
+      if( charset.enabled( Charset::iso_8859_15 ) ||
+          charset.enabled( Charset::iso_8859_9 ) )
+        if( !lp.isconvex() && bp.isconvex() &&
+            b.seek_bottom( row, h.hcenter() ) < b.bottom() )
+          return UCS::SEACUTE;
       row = ( row + b.seek_bottom( row, h.hcenter() ) ) / 2;
       if( row < b.bottom() - 1 && !lp.isflats() &&
           b.seek_left( row, h.hcenter() ) <= b.left() ) return '4';
@@ -358,30 +409,37 @@ unsigned char Features::test_4ADQao() const throw()
     if( b.width() > 3 * h.width() )
       {
       int c = vscan()[b.vcenter() - b.top()];
-      if( c == 3 && h.top() < b.vcenter() && !lp.isconvex() ) return 'm';
-      if( c >= 4 ) return '@';
+      if( c == 3 && h.top() < b.vcenter() && h.bottom() > b.vcenter() &&
+          !lp.ispit() && !rp.ispit() && !bp.ispit() )
+        return 'm';
+      if( c == 4 && Ocrad::similar( left_delta, right_delta, 40 ) &&
+          lp.ispit() ) return '@';
       }
     if( tp.minima() == 1 && bp.minima() == 2 && !lp.isconvex() ) return 'A';
     }
-  if( Ocrad::similar( h.left() - b.left(), b.right() - h.right(), 40 ) )
+  if( Ocrad::similar( left_delta, right_delta, 40 ) )
     {
-    if( !lp.isconvex() && !bp.isconvex() && !rp.isflat() && bp[1] < 2 )
-      return 'D';
+    if( !bp.isconvex() && rp.ispit() && bp[1] < 2 ) return 'D';
     if( bp.minima() > 1 || rp.minima() > 1 ) return 'Q';
+    if( b.bottom() < charbox_vcenter ) return UCS::DEG;
     return 'o';
     }
-  if( h.left() - b.left() > b.right() - h.right() && rp.ispit() ) return 'D';
-  if( bp.minima() > 1 || rp.minima() > 1 ) return 'a';
+  if( left_delta > right_delta && rp.ispit() && tp.minima() == 1 ) return 'D';
+  if( Ocrad::similar( left_delta, right_delta, 50 ) &&
+      ( bp.minima() > 1 || rp.minima() > 1 ) ) return 'a';
   return 0;
   }
 
 
 // Tests if the upper half of character is open to the left, to the right,
 // and/or to the bottom
-unsigned char Features::test_6abd() const throw()
+int Features::test_6abd( const Charset & charset ) const throw()
   {
   const Block & b = *_block;
-  const Block & h = b.block_vector().front();		// hole
+  const Block & h = b.block( 0 );			// hole
+
+  if( 3 * h.width() < b.width() &&
+      ( bp.minima( b.height() / 4 ) != 1 || tp.isctip() ) ) return 0;
 
   int col = h.hcenter();
   int row = b.seek_top( h.top(), col, false ) - 1;
@@ -401,12 +459,14 @@ unsigned char Features::test_6abd() const throw()
     int c = lp[urow-b.top()];
     bool oacute2 = ( c > lp[h.top()-b.top()] + 1 &&
                      tp[std::min(c-1,b.width()/4)] + b.top() >= urow );
-    if( oacute1 && oacute2 )
-      {
-      bool oacute3 = ( b.seek_left( b.top() + 1, b.right() + 1 ) >= rcol ||
-                       b.seek_right( h.top(), b.left() - 1 ) <= b.left() + 1 );
-      if( oacute3 ) return ISO_8859_1::SOACUTE;
-      }
+    if( charset.enabled( Charset::iso_8859_15 ) ||
+        charset.enabled( Charset::iso_8859_9 ) )
+      if( oacute1 && oacute2 )
+        {
+        bool oacute3 = ( b.seek_left( b.top() + 1, b.right() + 1 ) >= rcol ||
+                         b.seek_right( h.top(), b.left() - 1 ) <= b.left() + 1 );
+        if( oacute3 ) return UCS::SOACUTE;
+        }
     if( !oacute2 && lp.ispit() && bp.ispit() )
       {
       int row2 = b.seek_top( h.top(), h.right() + 1, false ) - 1;
@@ -423,8 +483,16 @@ unsigned char Features::test_6abd() const throw()
     int row2 = b.seek_top( h.top(), h.left(), false ) - 1;
     row2 = b.seek_top( row2, h.left() );
     if( row2 > b.top() )
-      { if( b.vcenter() < row ) return ISO_8859_1::SAACUTE; else return 'a'; }
-    if( oacute1 ) return ISO_8859_1::SOACUTE; else return 'd';
+      {
+      if( charset.enabled( Charset::iso_8859_15 ) ||
+          charset.enabled( Charset::iso_8859_9 ) )
+        if( b.vcenter() < row ) return UCS::SAACUTE;
+      return 'a';
+      }
+    if( charset.enabled( Charset::iso_8859_15 ) ||
+        charset.enabled( Charset::iso_8859_9 ) )
+      if( oacute1 ) return UCS::SOACUTE;
+    return 'd';
     }
 
   if( b.width() > 3 * h.width() && h.top() < b.vcenter() &&
@@ -434,7 +502,7 @@ unsigned char Features::test_6abd() const throw()
   }
 
 
-unsigned char Features::test_CEFIJLlT() const throw()
+int Features::test_CEFIJLlT( const Charset & charset ) const throw()
   {
   const Block & b = *_block;
 
@@ -465,7 +533,8 @@ unsigned char Features::test_CEFIJLlT() const throw()
       if( hbars() == 1 && 4 * hbar(0).height() <= b.height() )
         {
         if( hbar(0).top() <= b.top() + 1 ) return 'T';
-        if( std::abs( hbar(0).vcenter() - b.vcenter() ) <= 1 ) return '+';
+        if( std::abs( hbar(0).vcenter() - b.vcenter() ) <= 1 &&
+            Ocrad::similar( b.height(), b.width(), 50 ) ) return '+';
         }
       if( hbars() == 2 && 4 * hbar(0).height() <= b.height() &&
           hbar(0).top() <= b.top() + 1 &&
@@ -516,15 +585,19 @@ unsigned char Features::test_CEFIJLlT() const throw()
         hbars() == 1 && hbar(0).top() <= b.top() + 1 &&
         hbar(0).width() + 1 >= b.width() )
       {
-      if( b.width() > b.height() ) return ISO_8859_1::NOT;
+      if( charset.enabled( Charset::iso_8859_15 ) ||
+          charset.enabled( Charset::iso_8859_9 ) )
+        if( b.width() > b.height() ) return UCS::NOT;
       return 0;
       }
     }
 
-  if( vbars() == 1 && vbar(0).width() >= 2 )
+  if( vbars() == 1 && vbar(0).width() >= 2 &&
+      tp.minima() == 1 && bp.minima() == 1 )
     {
     if( 3 * b.height() > 4 * b.width() &&
-        std::abs( vbar(0).hcenter() - b.hcenter() ) <= 1 )
+        Ocrad::similar( vbar(0).left() - b.left(),
+                        b.right() - vbar(0).right(), 30, 2 ) )
       {
       if( rp.istip() && ( lp.istip() || lp.isflats() ) )
         {
@@ -571,18 +644,21 @@ unsigned char Features::test_CEFIJLlT() const throw()
       }
     }
   if( hbars() == 1 && std::abs( hbar(0).vcenter() - b.vcenter() ) <= 1 &&
-      tp.isupit() && bp.isupit() ) return '+';
+      Ocrad::similar( b.height(), b.width(), 50 ) &&
+      tp.isupit() && bp.isupit() )
+    return '+';
   return 0;
   }
 
 
-unsigned char Features::test_c() const throw()
+int Features::test_c() const throw()
   {
   const Block & b = *_block;
 
   if( lp.isconvex() )
     {
-    if( b.height() > 2 * b.width() ) return '(';
+    if( b.height() > 2 * b.width() )
+      { if( 2 * b.width() >= 3 * wp.max() ) return '('; else return 0; }
     if( b.seek_bottom( b.vcenter(), b.hcenter() ) < b.bottom() &&
         b.escape_right( b.vcenter(), b.hcenter() ) ) return 'c';
     }
@@ -593,7 +669,7 @@ unsigned char Features::test_c() const throw()
   }
 
 
-unsigned char Features::test_frst() const throw()
+int Features::test_frst() const throw()
   {
   const Block & b = *_block;
 
@@ -619,7 +695,8 @@ unsigned char Features::test_frst() const throw()
           tp.iminimum() > tp.pos( 50 ) && bp.iminimum() <= bp.pos( 50 ) &&
           ( !b_hdiff || rp.increasing( rp.pos( 50 ) ) ) )
         return 'f';
-      if( 2 * b_hdiff > b.height() && 2 * t_hdiff > b.height() ) return 's';
+      if( 2 * b_hdiff > b.height() && 2 * t_hdiff > b.height() )
+        return 0;	// recognized 's' or SCCEDI
       }
     }
 
@@ -641,7 +718,7 @@ unsigned char Features::test_frst() const throw()
           int col2 = b.seek_right( row2, b.hcenter(), false ) + 1;
           if( b.seek_right( row2, col2 ) >= b.right() )
             {
-            if( lp.isconvex() && ( col > b.hcenter() || row < b.bottom() ) )
+            if( lp.isconvex() && ( col > b.hpos(60) || row < b.bottom() ) )
               return 0;
             if( hbars() == 1 && hbar(0).top() <= b.top() + 1 &&
                 4 * hbar(0).height() <= b.height() &&
@@ -669,7 +746,8 @@ unsigned char Features::test_frst() const throw()
         if( rp.iminimum() < rp.pos(20) )
           {
           if( rp.increasing( rp.pos(20) ) || bp.increasing() ||
-              tp.minima( 1 ) == 2 ) return 'r';
+              tp.minima( 1 ) == 2 ||
+              ( rp.minima() == 1 && tp.iminimum() > tp.pos(50) ) ) return 'r';
           else return 'f';
           }
         else return 't';
@@ -682,7 +760,7 @@ unsigned char Features::test_frst() const throw()
   }
 
 
-unsigned char Features::test_G() const throw()
+int Features::test_G() const throw()
   {
   const Block & b = *_block;
 
@@ -719,7 +797,8 @@ unsigned char Features::test_G() const throw()
   }
 
 
-unsigned char Features::test_HKMNUuvwYy() const throw()
+// Common feature: U-shaped top of character
+int Features::test_HKMNUuvwYy() const throw()
   {
   const Block & b = *_block;
 
@@ -757,26 +836,28 @@ unsigned char Features::test_HKMNUuvwYy() const throw()
       if( lg > 1 && bp.ispit() && tp.minima( ( 2 * b.height() ) / 5 ) == 2 )
         return 'v';
       if( lg <= 1 && 2 * ( b.width() - rg - lg ) < b.width() &&
-          rp.increasing() && tp.minima( b.height()/2 ) == 2 ) return 'v';
+          rp.increasing() && tp.minima( b.height() / 2 ) == 2 ) return 'v';
       return 0;
       }
-    if( bp.minima( b.height() / 4 ) == 2 && bp.isctip() )
+    if( bp.minima() == 2 && bp.isctip() )
       {
-      if( vscan()[rp.pos(40)] == 4 ) return 'M';
+      if( bp.minima( bp[bp.pos(50)] + 1 ) == 3 ) return 'M';
       int lg = lp[lp.pos(50)];
       int rg = rp[rp.pos(50)];
-      if( Ocrad::similar( lg, rg, 80 ) && lg < b.width() / 4 && rg < b.width() / 4 )
+      if( Ocrad::similar( lg, rg, 80, 2 ) &&
+          lg < b.width() / 4 && rg < b.width() / 4 )
         {
         if( lg > 1 && rg > 1 && lp.increasing() && rp.increasing() )
           return 'w';
-        if( hbars() == 1 && std::abs( hbar(0).vcenter() - b.vcenter() ) <= 1 &&
-            std::abs( hbar(0).hcenter() - b.hcenter() ) <= 1 )
+        if( hbars() == 1 && 5 * hbar(0).height() < b.height() &&
+            hbar(0).v_includes( b.vcenter() ) &&
+            10 * hbar(0).h_distance( b.hcenter() ) <= b.width() )
           return 'H';
         if( vscan()[rp.pos(60)] == 4 || vscan()[rp.pos(70)] == 4 ) return 'w';
         if( b.width() > 2 * b.height() ) return '~'; else return 'N';
         }
       if( 3 * lg < 2 * rg && lg < b.width() / 4 && rg > b.width() / 4 &&
-          rp.isctip() ) return 'K';
+          rp.isctip() && tp.minima( b.height() / 8 ) == 2 ) return 'K';
       return 0;
       }
     if( bp.minima() == 3 ) return 'M';
@@ -787,7 +868,7 @@ unsigned char Features::test_HKMNUuvwYy() const throw()
 
 // Looks for the nearest frontier in column hcenter(), then tests if
 // gap is open downwards (except for 'x')
-unsigned char Features::test_hknwx() const throw()
+int Features::test_hknwx() const throw()
   {
   const Block & b = *_block;
 
@@ -826,14 +907,13 @@ unsigned char Features::test_hknwx() const throw()
 bool Features::test_comma() const throw()
   {
   const Block & b = *_block;
-  const Blockmap & bm = *_blockmap;
 
-  if( b.block_vector().size() > 0 ) return 0;
-  if( 4 * b.height() < 5 * b.width()) return false;
-  if( b.height() > 3 * b.width() ) return false;
+  if( b.blocks() || b.height() <= b.width() || b.height() > 3 * b.width() )
+    return false;
 
   if( b.width() >= 3 && b.height() >= 3 )
     {
+    const Blockmap & bm = *_blockmap;
     int upper_area = 0;
     for( int row = b.top(); row < b.top() + b.width(); ++row )
       for( int col = b.left(); col <= b.right(); ++col )
@@ -849,11 +929,11 @@ bool Features::test_comma() const throw()
   }
 
 
-unsigned char Features::test_easy( int charbox_vcenter ) const throw()
+int Features::test_easy( int charbox_vcenter ) const throw()
   {
   const Block & b = *_block;
-  unsigned char ch = test_solid( charbox_vcenter );
-  if( ch ) return ch;
+  int code = test_solid( charbox_vcenter );
+  if( code ) return code;
 
   if( b.top() >= charbox_vcenter && test_comma() ) return ',';
   if( b.bottom() <= charbox_vcenter &&
@@ -870,7 +950,7 @@ unsigned char Features::test_easy( int charbox_vcenter ) const throw()
 
 // Recognizes single line, non-rectangular characters without holes.
 // '/<>\^`
-unsigned char Features::test_line( int charbox_vcenter ) const throw()
+int Features::test_line( int charbox_vcenter ) const throw()
   {
   const Block & b = *_block;
   int slope1, slope2;
@@ -882,8 +962,8 @@ unsigned char Features::test_line( int charbox_vcenter ) const throw()
       if( slope2 < 0 ) { if( bp.minima() == 2 ) return '^'; else return 0; }
       if( b.v_includes( charbox_vcenter ) )
         {
-        if( 4 * b.area() < b.height() * b.width() ) return '/';
-        if( b.height() > b.width() ) return 'l';
+        if( 4 * b.area() < b.size() ) return '/';
+        if( b.height() > 2 * b.width() ) return 'l';
         return 0;
         }
       if( b.top() >= charbox_vcenter ) return ',';
@@ -891,11 +971,13 @@ unsigned char Features::test_line( int charbox_vcenter ) const throw()
       }
     if( slope1 > 0 && slope2 < 0 )
       {
-      if( b.bottom() > charbox_vcenter ) return '\\';
+      if( b.bottom() > charbox_vcenter )
+        { if( 2 * b.width() >= b.height() ) return '\\'; else return 0; }
       return '`';
       }
     }
-  if( tp.straight( &slope1 ) && bp.straight( &slope2 ) )
+  else if( 2 * b.width() >= b.height() && tp.straight( &slope1 ) &&
+           bp.straight( &slope2 ) )
     {
     if( slope1 < 0 && slope2 < 0 ) return '<';
     if( slope1 > 0 && slope2 > 0 ) return '>';
@@ -904,56 +986,60 @@ unsigned char Features::test_line( int charbox_vcenter ) const throw()
   }
 
 
-unsigned char Features::test_solid( int charbox_vcenter ) const throw()
+int Features::test_solid( int charbox_vcenter ) const throw()
   {
   const Block & b = *_block;
   if( b.blocks() ) return 0;
+
+  if( b.height() > 3 && ( tp.minima() != 1 || bp.minima() != 1 ) ) return 0;
+  if( b.width() > 3 && ( lp.minima() != 1 || rp.minima() != 1 ) ) return 0;
+
   const Blockmap & bm = *_blockmap;
-  if( bm.id( b.vcenter(), b.hcenter() ) != b.id() ) return 0;
-  int inner_area, inner_space;
+  int inner_area, inner_size, porosity = 0;
 
   if( b.width() >= 3 && b.height() >= 3 )
     {
-    inner_space = ( b.width() - 2 ) * ( b.height() - 2 );
+    inner_size = ( b.width() - 2 ) * ( b.height() - 2 );
     inner_area = 0;
     for( int row = b.top() + 1; row < b.bottom(); ++row )
       {
       int holes = 0;	// FIXME
       for( int col = b.left() + 1; col < b.right(); ++col )
         { if( bm.id( row, col ) == b.id() ) ++inner_area; else ++holes; }
-      if( 5 * holes >= b.width() ) return 0;
+      if( 5 * holes >= b.width() ) ++porosity;
       }
-    if( inner_area * 100 < inner_space * 75 ) return 0;
+    if( inner_area * 100 < inner_size * 70 ) return 0;
     }
-  else { inner_space = 0; inner_area = b.area(); }
+  else { inner_size = 0; inner_area = b.area(); }
 
-  if( std::abs( b.height() - wp.max() ) <= 2 ||
-      Ocrad::similar( b.height(), wp.max(), 20 ) ) return '.';
-  if( inner_area * 100 < inner_space * 85 ) return 0;
-  if( b.width() >= 6 * b.height() &&
-      b.top() - b.height() > charbox_vcenter ) return '_';
-  if( b.width() > b.height() ) return '-';
-
-  if( b.height() > b.width() && tp.minima() == 1 && bp.minima() == 1 )
+  if( Ocrad::similar( b.height(), wp.max(), 20, 2 ) )
     {
-    if( b.height() > 9 && b.width() <= 8 && b.v_includes( charbox_vcenter ) )
+    int n = std::min( b.height(), b.width() );
+    if( n >= 6 )
       {
-      int l = 0, r = 0;
-      for( int row = b.vcenter() - 1; row <= b.vcenter() + 1; ++row )
-        { if( bm.id( row, b.left() ) == b.id() ) ++l;
-        if( bm.id( row, b.right() ) == b.id() ) ++r; }
-      if( l <= 1 && r <= 1 )
+      int d = 0;
+      for( int i = 0; i < n; ++i )
         {
-        bool lt = false, lb = false, rt = false, rb = false;
-        for( int row = b.top(); row <= b.top() + 2; ++row )
-          { if( bm.id( row, b.left() ) == b.id() ) lt = true;
-          if( bm.id( row, b.right() ) == b.id() ) rt = true; }
-        for( int row = b.bottom() - 2; row <= b.bottom(); ++row )
-          { if( bm.id( row, b.left() ) == b.id() ) lb = true;
-          if( bm.id( row, b.right() ) == b.id() ) rb = true; }
-        if( lt && lb && rb ) { if( rt ) return 'I'; else return 'l'; }
+        if( bm.id( b.top() + i, b.left() + i ) == b.id() ) ++d;
+        if( bm.id( b.top() + i, b.right() - i ) == b.id() ) --d;
         }
+      if( 2 * std::abs( d ) >= n - 1 ) return 0;
       }
+    if( ( !porosity && inner_area * 100 >= inner_size * 75 ) ||
+        ( b.width() >= 7 && b.height() >= 7 &&
+          100 * b.area_octagon() >= 98 * b.area() ) ) return '.';
+    return 0;
+    }
+  if( porosity || inner_area * 100 < inner_size * 85 ) return 0;
+  if( b.width() > b.height() )
+    {
+    if( b.width() >= 6 * b.height() &&
+        b.top() - b.height() > charbox_vcenter ) return '_';
+    return '-';
+    }
+
+  if( b.height() > b.width() )
+    {
     if( b.top() > charbox_vcenter ) return ',';
     if( b.bottom() <= charbox_vcenter ) return '\'';
     return '|';
@@ -962,11 +1048,12 @@ unsigned char Features::test_solid( int charbox_vcenter ) const throw()
   }
 
 
-unsigned char Features::test_misc() const throw()
+int Features::test_misc() const throw()
   {
   const Block & b = *_block;
 
-  if( hbars() == 1 && hbar(0).top() <= b.top() + (b.height() / 10) &&
+  if( hbars() == 1 && hbar(0).top() <= b.top() + ( b.height() / 10 ) &&
+      4 * hbar(0).height() <= b.height() &&
       5 * hbar(0).width() >= 4 * b.width() &&
       rp.increasing( hbar(0).vcenter() - b.top() ) &&
       rp[hbar(0).bottom()-b.top()+2] - rp[hbar(0).bottom()-b.top()] < b.width() / 4 )
@@ -1005,10 +1092,12 @@ unsigned char Features::test_misc() const throw()
             3 * hbar(0).width() > 4 * hbar(2).width() ) ) ) return 'F';
     }
 
-  if( b.height() > 2 * wp.max() )
+  if( b.height() >= 3 * wp.max() )
     {
-    if( rp.istip() && lp.ispit() ) return '{';
-    if( lp.istip() && rp.ispit() ) return '}';
+    if( rp.istip() && lp.ispit() )
+      { if( lp.isvpit() ) return '{'; else return '('; }
+    if( lp.istip() && rp.ispit() )
+      { if( rp.isvpit() ) return '}'; else return ')'; }
     }
 
   return 0;
