@@ -1,5 +1,5 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
 
 class Features
@@ -41,20 +41,21 @@ public:
   const std::vector< int > & hscan() const throw();
   const std::vector< int > & vscan() const throw();
 
-  unsigned char test_235Esz() const throw();
-  unsigned char test_49ARegpq() const throw();
-  unsigned char test_4ADQao() const throw();
-  unsigned char test_6abd() const throw();
-  unsigned char test_CEFIJLlT() const throw();
-  unsigned char test_c() const throw();
-  unsigned char test_frst() const throw();
-  unsigned char test_G() const throw();
-  unsigned char test_HKMNUuvwYy() const throw();
-  unsigned char test_hknwx() const throw();
+  int test_235Esz( const Charset & charset ) const throw();
+  int test_49ARegpq() const throw();
+  int test_4ADQao( const Charset & charset, int charbox_vcenter ) const throw();
+  int test_6abd( const Charset & charset ) const throw();
+  int test_CEFIJLlT( const Charset & charset ) const throw();
+  int test_c() const throw();
+  int test_frst() const throw();
+  int test_G() const throw();
+  int test_HKMNUuvwYy() const throw();
+  int test_hknwx() const throw();
+  int test_s_cedilla() const throw();
 
   bool test_comma() const throw();
-  unsigned char test_easy( int charbox_vcenter ) const throw();
-  unsigned char test_line( int charbox_vcenter ) const throw();
-  unsigned char test_solid( int charbox_vcenter ) const throw();
-  unsigned char test_misc() const throw();
+  int test_easy( int charbox_vcenter ) const throw();
+  int test_line( int charbox_vcenter ) const throw();
+  int test_solid( int charbox_vcenter ) const throw();
+  int test_misc() const throw();
   };
