@@ -1,5 +1,5 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
 /*
     Return values: 0 for a normal exit, 1 for environmental problems
@@ -40,12 +40,13 @@
 
 namespace {
 
-// Date of this version: 2003-12-18
+// Date of this version: 2004-02-09
 
-const char *const program_version = "0.6";
-const char *const program_year    = "2003";
+const char *const program_version = "0.7";
+const char *const program_year    = "2004";
 
-void show_version()
+
+void show_version() throw()
   {
   std::printf( "GNU Ocrad version %s\n", program_version );
   std::printf( "Copyright (C) %s Antonio Diaz.\n", program_year );
@@ -53,23 +54,28 @@ void show_version()
   std::printf( "the GNU General Public License.  This program has absolutely no warranty.\n" );
   }
 
-void show_error( char * msg )
+
+void show_error( const char * program_name, const char * msg ) throw()
   {
   if( msg && msg[0] != 0 ) std::fprintf( stderr,"ocrad: %s\n", msg );
-  std::fprintf( stderr,"Try `ocrad --help' for more information.\n" );
+  if( program_name && program_name[0] != 0 )
+    std::fprintf( stderr,"Try `%s --help' for more information.\n", program_name );
   }
 
-void show_help( bool verbose )
+
+void show_help( const char * program_name, bool verbose ) throw()
   {
   std::printf( "GNU Ocrad, Optical Character Recognition program\n" );
   std::printf( "Reads pbm file(s), or standard input, and sends text to standard output\n" );
-  std::printf( "\nUsage: ocrad [options] [files]\n" );
+  std::printf( "\nUsage: %s [options] [files]\n", program_name );
   std::printf( "Options:\n");
   std::printf( "  -h, --help               display this help and exit\n");
   std::printf( "  -V, --version            output version information and exit\n");
   std::printf( "  -a, --append             append text to output file\n");
   std::printf( "  -b, --block=<n>          process only the specified text block\n");
+  std::printf( "  -c, --charset=<name>     try `--charset=help' for a list of names\n");
   std::printf( "  -f, --force              force overwrite of output file\n");
+  std::printf( "  -F, --format=<fmt>       output format (byte, utf8)\n");
   std::printf( "  -i, --invert             invert image levels (white on black)\n");
   std::printf( "  -l, --layout=<n>         layout analysis, 0=none, 1=column, 2=full\n");
   std::printf( "  -o <file>                place the output into <file>\n");
@@ -89,96 +95,94 @@ void show_help( bool verbose )
   }
 
 
-void ocr( const Bitmap & page_image, FILE * outfile, FILE * exportfile,
-          int rindex, int debug_level ) throw()
+void ocr( const Bitmap & page_image, const Charset & charset,
+                                     const Control & control, int c ) throw()
   {
-  Blockmap blockmap( page_image, rindex, debug_level );
+  Blockmap blockmap( page_image, c, control.debug_level );
   if( !blockmap.blocks() ) return;
 
-  if( debug_level >= 90 )
-    { blockmap.print( outfile, debug_level ); return; }
+  if( control.debug_level >= 90 )
+    { blockmap.print( control.outfile, control.debug_level ); return; }
 
-  Textblock textblock( blockmap.block_vector() );
+  Textblock textblock( charset, blockmap.block_vector() );
 
-  if( debug_level >= 86 )
+  if( control.debug_level >= 86 )
     {
-    bool graph = ( debug_level >= 88 );
-    bool recursive = ( debug_level & 1 );
-    textblock.print( outfile, graph, recursive );
+    bool graph = ( control.debug_level >= 88 );
+    bool recursive = ( control.debug_level & 1 );
+    textblock.dprint( control, graph, recursive );
     }
 
-  if( debug_level == 0 ) textblock.print( outfile, false, false );
-  if( exportfile ) textblock.xprint( exportfile );
+  if( control.debug_level == 0 ) textblock.print( control );
+  if( control.exportfile ) textblock.xprint( control );
   }
 
 
-int process_file( FILE *infile, FILE *outfile, FILE *exportfile, int rindex,
-                  int debug_level, int layout_level, int scale, int threshold,
-                  bool copy, bool invert, bool level, bool verbose,
-                  char filetype, char specialtype )
+int process_file( FILE *infile, const Charset & charset,
+                                const Control & control ) throw()
   {
   try
     {
-    Bitmap page_image( infile, invert );
+    Bitmap page_image( infile, control.invert );
 
-    page_image.analyse_layout( layout_level );
-    if( verbose ) std::fprintf( stderr, "number of text blocks %d\n",
+    page_image.analyse_layout( control.layout_level );
+    if( control.verbose ) std::fprintf( stderr, "number of text blocks %d\n",
                                 page_image.rectangles() );
 
-    if( rindex >= page_image.rectangles() )
+    if( control.rindex >= page_image.rectangles() )
       {
       std::fprintf( stderr,"This page has only %d text block(s)\n",
                     page_image.rectangles() );
       return 1;
       }
 
-    if( level )
+    if( control.level )
       std::fprintf( stderr, "\nslope_best = %d\n",
-                    page_image.horizontalify( verbose ) );
+                    page_image.horizontalify( control.verbose ) );
 
-    if( scale != 0 )
+    if( control.scale != 0 )
       {
-      Bitmap reduced( page_image, scale, threshold );
-      reduced.save( outfile, filetype ); return 0;
+      Bitmap reduced( page_image, control.scale, control.threshold );
+      reduced.save( control.outfile, control.filetype ); return 0;
       }
-    else if( specialtype != 0 )
+    else if( control.specialtype != 0 )
       {
       Bitmap::type t;
-      if( specialtype == 'v' ) t = Bitmap::vertical_histogram ;
-      else if( specialtype == 'h' ) t = Bitmap::horizontal_histogram ;
-      else if( specialtype == 'g' ) t = Bitmap::connected_ground ;
-      else { show_error( "bad special type" ); return 1; }
+      if( control.specialtype == 'v' ) t = Bitmap::vertical_histogram ;
+      else if( control.specialtype == 'h' ) t = Bitmap::horizontal_histogram ;
+      else if( control.specialtype == 'g' ) t = Bitmap::connected_ground ;
+      else { show_error( 0, "bad special type" ); return 1; }
       Bitmap sb( page_image, t );
-      sb.save( outfile, filetype ); return 0;
+      sb.save( control.outfile, control.filetype ); return 0;
       }
-    else if( copy )
+    else if( control.copy )
       {
-      if( rindex < 0 && page_image.rectangles() == 1 )
-        { page_image.save( outfile, filetype ); return 0; }
+      if( control.rindex < 0 && page_image.rectangles() == 1 )
+        { page_image.save( control.outfile, control.filetype ); return 0; }
       for( int c = 0; c < page_image.rectangles(); ++c )
-        if( rindex < 0 || rindex == c )
+        if( control.rindex < 0 || control.rindex == c )
           {
           Bitmap bitmap( page_image, page_image.rectangle_vector()[c] );
-          bitmap.save( outfile, filetype );
+          bitmap.save( control.outfile, control.filetype );
           }
       return 0;
       }
 
-    if( exportfile )
-      std::fprintf( exportfile, "total blocks %d\n",
-                    ( rindex < 0 ) ? page_image.rectangles() : 1 );
+    if( control.exportfile )
+      std::fprintf( control.exportfile, "total blocks %d\n",
+                    ( control.rindex < 0 ) ? page_image.rectangles() : 1 );
 
     // call the character recognizer for every rectangle of text
     for( int c = 0; c < page_image.rectangles(); ++c )
-      if( rindex < 0 || rindex == c )
+      if( control.rindex < 0 || control.rindex == c )
         {
-        if( exportfile )
+        if( control.exportfile )
           {
           const Rectangle & r = page_image.rectangle_vector()[c];
-          std::fprintf( exportfile, "block %d %d %d %d %d\n", c + 1,
+          std::fprintf( control.exportfile, "block %d %d %d %d %d\n", c + 1,
                         r.left(), r.top(), r.width(), r.height() );
           }
-        ocr( page_image, outfile, exportfile, c, debug_level );
+        ocr( page_image, charset, control, c );
         }
     }
   catch( Bitmap::Error e ) { std::fputs( e.s, stderr ); return 2; }
@@ -186,7 +190,7 @@ int process_file( FILE *infile, FILE *outfile, FILE *exportfile, int rindex,
   }
 
 
-const char * my_basename( const char * filename )
+const char * my_basename( const char * filename ) throw()
   {
   const char * c = filename;
   while( *c ) { if( *c == '/' ) filename = c + 1; ++c; }
@@ -201,15 +205,11 @@ const char * my_basename( const char * filename )
 // 'outfile' is the destination for the text version of the scanned
 // image. (or for a pbm file if debugging).
 // 'exportfile' is the Ocr Results File.
-int main( int argc, char *argv[] ) 
+int main( int argc, char *argv[] ) throw()
   {
-  FILE *outfile = stdout, *exportfile = 0;
+  Charset charset;
+  Control control;
   char *outfile_name = 0, *exportfile_name = 0;
-  int rindex = -1, debug_level = 0, layout_level = 0;
-  int scale = 0, threshold = -1;
-  char filetype = 0, specialtype = 0;
-  bool append = false, copy = false, force = false, invert = false;
-  bool level = false, verbose = false;
 
   // scan arguments
 
@@ -219,9 +219,11 @@ int main( int argc, char *argv[] )
       {
       {"append", no_argument, 0, 'a'},
       {"block", required_argument, 0, 'b'},
+      {"charset", required_argument, 0, 'c'},
       {"copy", no_argument, 0, 'C'},
       {"debug", required_argument, 0, 'D'},
       {"force", no_argument, 0, 'f'},
+      {"format", required_argument, 0, 'F'},
       {"help", no_argument, 0, 'h'},
       {"invert", no_argument, 0, 'i'},
       {"layout", required_argument, 0, 'l'},
@@ -231,31 +233,37 @@ int main( int argc, char *argv[] )
       {0, 0, 0, 0}
       };
 
-    int c = getopt_long( argc, argv, "14ab:CD:fhiLl:o:R:S:T:Vvx:",
+    int c = getopt_long( argc, argv, "14ab:c:CD:fF:hiLl:o:R:S:T:Vvx:",
                          long_options, 0 );
     if( c == -1 ) break;		// all options processed
 
     switch( c )
       {
       case 0: break;
-      case '?': return 1;  // `getopt_long' already printed an error message.
+      case '?': show_error( argv[0], 0 ); return 1;	// bad option
       case '1':
-      case '4': filetype = c; break;
-      case 'a': append = true; break;
-      case 'b': rindex = strtol( optarg, 0, 0 ) - 1; break;
-      case 'C': copy = true; break;
-      case 'D': debug_level = strtol( optarg, 0, 0 ); break;
-      case 'f': force = true; break;
-      case 'h': show_help( verbose ); return 0;
-      case 'i': invert = true; break;
-      case 'L': level = true; break;
-      case 'l': layout_level = strtol( optarg, 0, 0 ); break;
+      case '4': control.filetype = c; break;
+      case 'a': control.append = true; break;
+      case 'b': control.rindex = std::strtol( optarg, 0, 0 ) - 1; break;
+      case 'c': if( !charset.enable( optarg ) )
+                  { charset.show_error( optarg ); return 1; }
+                break;
+      case 'C': control.copy = true; break;
+      case 'D': control.debug_level = std::strtol( optarg, 0, 0 ); break;
+      case 'f': control.force = true; break;
+      case 'F': if( !control.set_format( optarg ) )
+                  { show_error( argv[0], "bad output format" ); return 1; }
+                break;
+      case 'h': show_help( argv[0], control.verbose ); return 0;
+      case 'i': control.invert = true; break;
+      case 'l': control.layout_level = std::strtol( optarg, 0, 0 ); break;
+      case 'L': control.level = true; break;
       case 'o':	outfile_name = optarg; break;
-      case 'R': scale = strtol( optarg, 0, 0 ); break;
-      case 'S': specialtype = optarg[0]; break;
-      case 'T': threshold = strtol( optarg, 0, 0 ); break;
+      case 'R': control.scale = std::strtol( optarg, 0, 0 ); break;
+      case 'S': control.specialtype = optarg[0]; break;
+      case 'T': control.threshold = std::strtol( optarg, 0, 0 ); break;
+      case 'v': control.verbose = true; break;
       case 'V':	show_version(); return 0;
-      case 'v': verbose = true; break;
       case 'x':	exportfile_name = optarg; break;
       default: return 1;
       }
@@ -263,23 +271,24 @@ int main( int argc, char *argv[] )
 
   if( outfile_name )
     {
-    if( append ) outfile = std::fopen( outfile_name, "a" );
-    else if( force ) outfile = std::fopen( outfile_name, "w" );
-    else if( ( outfile = std::fopen( outfile_name, "wx" ) ) == 0 )
+    if( control.append ) control.outfile = std::fopen( outfile_name, "a" );
+    else if( control.force ) control.outfile = std::fopen( outfile_name, "w" );
+    else if( ( control.outfile = std::fopen( outfile_name, "wx" ) ) == 0 )
       {
       std::fprintf( stderr, "Output file %s already exists.\n", outfile_name );
       return 1;
       }
-    if( !outfile )
+    if( !control.outfile )
       { std::fprintf( stderr, "Cannot open %s\n", outfile_name ); return 1; }
     }
 
   if( exportfile_name )
     {
-    exportfile = std::fopen( exportfile_name, "w" );
-    if( !exportfile )
+    control.exportfile = std::fopen( exportfile_name, "w" );
+    if( !control.exportfile )
       { std::fprintf( stderr, "Cannot open %s\n", exportfile_name ); return 1; }
-    std::fprintf( exportfile, "# Ocr Results File. Created by GNU Ocrad version %s\n",
+    std::fprintf( control.exportfile,
+                  "# Ocr Results File. Created by GNU Ocrad version %s\n",
                   program_version );
     }
 
@@ -299,7 +308,7 @@ int main( int argc, char *argv[] )
       if( infile ) std::fclose( infile );
       if( optind >= argc ) { infile = 0; break; }
       infile_name = argv[optind++];
-      if( strcmp( infile_name, "-" ) == 0 ) infile = stdin;
+      if( std::strcmp( infile_name, "-" ) == 0 ) infile = stdin;
       else infile = std::fopen( infile_name, "r" );
       if( infile ) break;
       std::fprintf( stderr, "Cannot open %s\n", infile_name );
@@ -307,18 +316,17 @@ int main( int argc, char *argv[] )
       }
     if( !infile ) break;
 
-    if( exportfile )
-      std::fprintf( exportfile, "source file %s\n", my_basename( infile_name ) );
+    if( control.exportfile )
+      std::fprintf( control.exportfile, "source file %s\n",
+                    my_basename( infile_name ) );
 
-    int tmp = process_file( infile, outfile, exportfile, rindex, debug_level,
-                            layout_level, scale, threshold, copy, invert,
-                            level, verbose, filetype, specialtype );
+    int tmp = process_file( infile, charset, control );
     if( tmp > 0 && infile == stdin ) infile = 0;
     if( tmp > retval ) retval = tmp;
-    std::fflush( outfile );
-    if( exportfile ) std::fflush( exportfile );
+    std::fflush( control.outfile );
+    if( control.exportfile ) std::fflush( control.exportfile );
     }
-  std::fclose( outfile );
-  if( exportfile ) std::fclose( exportfile );
+  std::fclose( control.outfile );
+  if( control.exportfile ) std::fclose( control.exportfile );
   return retval;
   }
