@@ -1,5 +1,5 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
 
 #include <cstdio>
@@ -315,7 +315,7 @@ bool Profile::ispit() throw()
     if( samples() < 5 ) return _ispit;
 
     int begin = 0, end = 0, i, ref;
-    for( i = 0, ref = limit(); i < samples(); ++i )
+    for( i = 0, ref = _limit; i < samples(); ++i )
       {
       int d = data[i];
       if( d == min() ) { begin = i; break; }
@@ -323,7 +323,7 @@ bool Profile::ispit() throw()
       }
     if( begin < 2 || begin > samples() - 3 ) return _ispit;
 
-    for( i = samples() - 1, ref = limit(); i >= begin; --i )
+    for( i = samples() - 1, ref = _limit; i >= begin; --i )
       {
       int d = data[i];
       if( d == min() ) { end = i; break; }
@@ -340,6 +340,30 @@ bool Profile::ispit() throw()
   }
 
 
+bool Profile::iscpit( int cpos ) throw()
+  {
+  if( !valid ) initialize();
+  if( samples() < 5 || cpos < 25 || cpos > 75 ) return false;
+  int th = ( mean() < 2 ) ? 2 : mean();
+  int imin = -1, mid = ( ( samples() - 1 ) * cpos ) / 100;
+
+  for( int i = 0; i < samples() / 4; ++i )
+    {
+    if( data[mid+i] < th ) { imin = mid + i; break; }
+    if( data[mid-i-1] < th ) { imin = mid - i - 1; break; }
+    }
+  if( imin < 0 ) return false;
+
+  for( int i = imin + 1; i < samples(); ++i )
+    if( data[i] > th )
+      {
+      for( int j = imin - 1; j >= 0; --j ) if( data[j] > th ) return true;
+      break;
+      }
+  return false;
+  }
+
+
 bool Profile::isupit() throw()
   {
   if( !valid_isupit )
@@ -384,12 +408,18 @@ bool Profile::isvpit() throw()
     valid_isvpit = true; if( !valid ) initialize();
     if( !ispit() ) { _isvpit = false; return _isvpit; }
 
-    int count1 = 0, count2 = 0;
+    int l1 = -1, l2 = -1, r1 = -2, r2 = -2;
     for( int i = 0; i < samples(); ++i )
-      if( data[i] <= 2 ) { if( data[i] < 2 ) ++count1; else ++count2; }
+      if( data[i] <= 2 )
+        {
+        r2 = i; if( l2 < 0 ) l2 = i;
+        if( data[i] <= 1 ) { r1 = i; if( l1 < 0 ) l1 = i; }
+        }
+    int count1 = r1 - l1 + 1, count2 = r2 - l2 + 1 - count1;
     _isvpit = ( 4 * count1 < samples() ||
                 ( 3 * count1 <= samples() && count2 <= 2 ) ||
-                ( 5 * count1 <= 2 * samples() && count2 <= 1 ) );
+                ( 5 * count1 <= 2 * samples() && count2 <= 1 ) ||
+                ( 2 * count1 <= samples() && count2 <= 0 ) );
     }
   return _isvpit;
   }
@@ -412,11 +442,12 @@ bool Profile::istip() throw()
     for( int i = 2; i < samples() - 2; ++i )
       switch( status )
         {
-        case 0: if( data[i] > th ) status = 1; break;
-        case 1: if( data[i] <= th ) status = 2; break;
-        case 2: if( data[i] > th ) return _istip;
+        case 0: if( data[i] > th + 1 ) status = 1; break;
+        case 1: if( data[i] > th + 1 ) status = 2; else status = 0; break;
+        case 2: if( data[i] <= th ) status = 3; break;
+        case 3: if( data[i] > th + 1 ) return _istip;
         }
-    _istip = ( status >= 1 );
+    _istip = ( status >= 2 );
     }
   return _istip;
   }
