@@ -1,5 +1,5 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
 
 class Rectangle
@@ -35,6 +35,7 @@ public:
 
   int height() const throw() { return _bottom - _top + 1; }
   int width()  const throw() { return _right - _left + 1; }
+  int size()   const throw() { return height() * width(); }
 
   int hcenter() const throw() { return ( _left + _right ) / 2; }
   int vcenter() const throw() { return ( _top + _bottom ) / 2; }
