@@ -1,5 +1,5 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
 
 #include <cstdio>
@@ -26,7 +26,7 @@
 #include "textblock.h"
 
 
-Textblock::Textblock( const std::vector< Block > & block_vector ) throw()
+Textblock::Textblock( const Charset & charset, const std::vector< Block > & block_vector ) throw()
   {
   // First pass. Assign blocks to characters and create lines.
   {
@@ -160,9 +160,9 @@ Textblock::Textblock( const std::vector< Block > & block_vector ) throw()
   for( int i = 0; i < lines(); ++i )
     {
     // First pass. Recognize the easy characters.
-    line(i).recognize1();
+    line(i).recognize1( charset );
     // Second pass. Use context to clear up ambiguities.
-    line(i).recognize2();
+    line(i).recognize2( charset );
     }
   }
 
@@ -199,29 +199,36 @@ Textline & Textblock::line( const int i ) const throw()
   }
 
 
-void Textblock::print( FILE * outfile, const bool graph, const bool recursive )
+void Textblock::print( const Control & control ) const throw()
+  {
+  for( int i = 0; i < lines(); ++i ) line(i).print( control );
+  std::fputs( "\n", control.outfile );
+  }
+
+
+void Textblock::dprint( const Control & control, bool graph, bool recursive )
 								const throw()
   {
-  if( graph || recursive ) std::fprintf( outfile, "%d lines\n\n", lines() );
+  std::fprintf( control.outfile, "%d lines\n\n", lines() );
 
   for( int i = 0; i < lines(); ++i )
     {
-    if( graph || recursive )
-      std::fprintf( outfile, "%d characters in line %d\n", line(i).characters(), i+1 );
-    line(i).print( outfile, graph, recursive );
+    std::fprintf( control.outfile, "%d characters in line %d\n",
+                  line(i).characters(), i+1 );
+    line(i).dprint( control, graph, recursive );
     }
-  std::fputs( "\n", outfile );
+  std::fputs( "\n", control.outfile );
   }
 
 
-void Textblock::xprint( FILE * exportfile ) const throw()
+void Textblock::xprint( const Control & control ) const throw()
   {
-  std::fprintf( exportfile, "lines %d\n", lines() );
+  std::fprintf( control.exportfile, "lines %d\n", lines() );
 
   for( int i = 0; i < lines(); ++i )
     {
-    std::fprintf( exportfile, "line %d chars %d height %d\n", i + 1,
+    std::fprintf( control.exportfile, "line %d chars %d height %d\n", i + 1,
                   line(i).characters(), line(i).mean_height() );
-    line(i).xprint( exportfile );
+    line(i).xprint( control );
     }
   }
