@@ -1,5 +1,5 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
 
 class Textblock
@@ -25,12 +25,13 @@ class Textblock
   void delete_line( int ) throw();
 
 public:
-  Textblock( const std::vector< Block > & block_vector ) throw();
+  Textblock( const Charset & charset, const std::vector< Block > & block_vector ) throw();
 
   Textline & line( int ) const throw();
   int lines() const throw() { return data.size(); }
   int characters() const throw();
 
-  void print( FILE * outfile, bool graph, bool recursive ) const throw();
-  void xprint( FILE * exportfile ) const throw();
+  void print( const Control & control ) const throw();
+  void dprint( const Control & control, bool graph, bool recursive ) const throw();
+  void xprint( const Control & control ) const throw();
   };
