@@ -1,5 +1,5 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
 
 #include <cctype>
@@ -21,9 +21,9 @@
 #include <vector>
 #include "common.h"
 #include "rectangle.h"
+#include "ucs.h"
 #include "block.h"
 #include "character.h"
-#include "iso_8859_1.h"
 #include "textline.h"
 
 
@@ -32,8 +32,8 @@ int Textline::find_space( int begin ) const throw()
   int end = begin;
   for( ; end < characters(); ++end )
     {
-    Character & c = character( end );
-    if( c.guesses() && std::isspace( c.guess(0).ch ) ) break;
+    const Character & c = character( end );
+    if( c.guesses() && UCS::isspace( c.guess(0).code ) ) break;
     }
   return end;
   }
@@ -158,25 +158,33 @@ void Textline::join( Textline & l ) throw()
   }
 
 
-void Textline::print( FILE * outfile, bool graph, bool recursive ) const throw()
+void Textline::print( const Control & control ) const throw()
+  {
+  for( int i = 0; i < characters(); ++i ) character( i ).print( control );
+  std::fputs( "\n", control.outfile );
+  }
+
+
+void Textline::dprint( const Control & control, bool graph, bool recursive )
+								const throw()
   {
   if( graph || recursive )
-    std::fprintf( outfile, "mean_height = %d\n", mean_height() );
+    std::fprintf( control.outfile, "mean_height = %d\n", mean_height() );
 
   for( int i = 0; i < characters(); ++i )
-    character( i ).print( outfile, graph, recursive );
-  std::fputs( "\n", outfile );
+    character( i ).dprint( control, graph, recursive );
+  std::fputs( "\n", control.outfile );
   }
 
 
-void Textline::xprint( FILE * exportfile ) const throw()
+void Textline::xprint( const Control & control ) const throw()
   {
   for( int i = 0; i < characters(); ++i )
-    character(i).xprint( exportfile );
+    character( i ).xprint( control );
   }
 
 
-void Textline::recognize1() const throw()
+void Textline::recognize1( const Charset & charset ) const throw()
   {
   if( characters() == 0 ) return;
   int mh = mean_height(), l, r;
@@ -203,6 +211,6 @@ void Textline::recognize1() const throw()
     int charbox_vcenter = yl;
     if( dx )
       charbox_vcenter += ( dy * ( c.hcenter() - xl ) / dx );
-    c.recognize1( charbox_vcenter );
+    c.recognize1( charset, charbox_vcenter );
     }
   }
