@@ -1,5 +1,5 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
 
 class Textline
@@ -29,8 +29,8 @@ public:
   void delete_character( int i ) throw();
   const Character & shift_character( const Character & c ) throw();
   void swap_characters( int i, int j ) throw();
-  Character & character( int i ) const throw();
 
+  Character & character( int i ) const throw();
   const Character & front() const throw() { return data.front(); }
   const Character & back() const throw() { return data.back(); }
   int characters() const throw() { return data.size(); }
@@ -43,9 +43,10 @@ public:
   int mean_vcenter() const throw();
 
   void join( Textline & l ) throw();
-  void print( FILE * outfile, bool graph, bool recursive ) const throw();
-  void xprint( FILE * exportfile ) const throw();
+  void print( const Control & control ) const throw();
+  void dprint( const Control & control, bool graph, bool recursive ) const throw();
+  void xprint( const Control & control ) const throw();
 
-  void recognize1() const throw();
-  void recognize2() throw();
+  void recognize1( const Charset & charset ) const throw();
+  void recognize2( const Charset & charset ) throw();
   };
